import 'package:flutter/widgets.dart';
import 'package:microfrontends/microfrontends.dart';
import 'package:perf_showcase/config.dart';

import 'state.dart';

class ColorGridStateBinding extends Binding<ColorGridState> {
  ColorGridStateBinding({required Widget child}) : super(child: child);

  @override
  StateContainer<ColorGridState> create(BuildContext context) {
    return container<ColorGridState>(ColorGridState()..generate())
      ..query<Color>((ctrl, state, params) => state.colors[params as int])
      ..mutation<GenerateColors>((ctrl, state, mutation) {
        ctrl.run(() async* {
          while (true) {
            await Future.delayed(pause);
            for (int i = 0; i < state.colorsPerIteration; i++) {
              try {
                state.getNextColor();
              } catch (err) {
                break;
              }
            }
            yield ColorGridState()
              ..colors = state.colors
              ..nonColored = state.nonColored;
            if (state.nonColored.length == 0) break;
          }
        });
        return state;
      });
  }
}
